function [InIndex, F]=BEEM(p1_m, p2_m, rr1, rr2, distanceRatio, N_BEEM)
% The demo contained in this package is described in 
% 
% Balanced Exploration and Exploitation Model Search for Efficient Epipolar Geometry Estimation
% Liran Goshen, Ilan Shimshoni
% in the proceedings of ECCV 2006.
% 
% Please refer to the paper for details
% 
% 
% The main function in this package is:
% 
% [InIndex, F]=BEEM(p1_m, p2_m, rr1, rr2, distanceRatio, N_BEEM)
% 
% Output:
% InIndex- Index of inliers
% F- Fundamental matrix
% 
% Input:
% p1_m, p2_m- putative match (Enclose are example files).
% rr1, rr2- SIFT parameters for the putative match. rr1 for p1_m and rr2 for p2_m. The first column in rr1/rr2 is the rotation angle of the dominant gradient of the SIFT descriptor. The second column is the scale in the space scale of the SIFT descriptor (Enclosed are example files). 
% distanceRatio - the distance ratio of the closest to the second-closest neighbors of the keypoint of the first image ($r_i$ in the paper). (Enclose is example file).
% N_BEEM- Maximum number of iteration (default value is 500).
% 
% Note:
% The program was written under MATLAB v6.1.
%

% Ngoal = stop alg when got the goal
Ngoal=99999;

% N_BEEM = max iter to run alg
if nargin<6, N_BEEM=500; end

% updatePin = 0 or 1
updatePin=1;

% NgrSam = 2 or 3 or 7 or 8
NgrSam=2;

% constraintAll3SIFTpoints = 0 or 1
constraintAll3SIFTpoints=1;

% MaxGoldIter=10 for exploitation stage
MaxGoldIter=10;

% LO = 0 or 1 => if 1 run improved explotitation with all point untill no improvment
LO=1;

% chkStatis = 0/1   => if to colect statistical information
chkStatis=0;

load outPDF;

gotPin=0;
multSIFTd=1;

probLiran=NinEstCDF(distanceRatio);
probLiran=probLiran/max(probLiran);

ransac_tr=5;
generateLength=7;


fprintf('BEEM algorithm:\n');


match=[p1_m, p2_m];


N=size(match,1);

[outCDF maxNoGrOut]=cr_outCDF(N,outPDF);

% Generate the dual points
match_g=zeros(N,4);
match_g1=zeros(N,4);
match_g2=zeros(N,4);

for i=1:N,
    x=generateLength*cos(rr1(i,1))*rr1(i,2)+p1_m(i,1);
    y=-generateLength*sin(rr1(i,1))*rr1(i,2)+p1_m(i,2);
    match_g(i,1)=x;
    match_g(i,2)=y;
    x=generateLength*cos(rr2(i,1))*rr2(i,2)+p2_m(i,1);
    y=-generateLength*sin(rr2(i,1))*rr2(i,2)+p2_m(i,2);
    match_g(i,3)=x;
    match_g(i,4)=y;
    
    ang=2*pi/3;
    x=generateLength*cos(rr1(i,1)+ang)*rr1(i,2)+p1_m(i,1);
    y=-generateLength*sin(rr1(i,1)+ang)*rr1(i,2)+p1_m(i,2);
    match_g1(i,1)=x;
    match_g1(i,2)=y;
    x=generateLength*cos(rr2(i,1)+ang)*rr2(i,2)+p2_m(i,1);
    y=-generateLength*sin(rr2(i,1)+ang)*rr2(i,2)+p2_m(i,2);
    match_g1(i,3)=x;
    match_g1(i,4)=y;

    ang=2*pi*2/3;
    x=generateLength*cos(rr1(i,1)+ang)*rr1(i,2)+p1_m(i,1);
    y=-generateLength*sin(rr1(i,1)+ang)*rr1(i,2)+p1_m(i,2);
    match_g2(i,1)=x;
    match_g2(i,2)=y;
    x=generateLength*cos(rr2(i,1)+ang)*rr2(i,2)+p2_m(i,1);
    y=-generateLength*sin(rr2(i,1)+ang)*rr2(i,2)+p2_m(i,2);
    match_g2(i,3)=x;
    match_g2(i,4)=y;
end



if (chkStatis),
    load allIn;
    load degIn;
    outDegIn=setdiff(allIn, degIn);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%% RANSAC 3

RANSACres=zeros(1,7);   % statistical res of ransac
iterDD=0;               % total number of iter
iter=0;                 % iter not include golden iters
iterGlobal=0;           % Number of global iterations
maxInGr_N=0;            % phase2 Number of max inliers
maxInGr=[];             % phase2 max inliers index
maxInGrCur_N=0;         % current
maxInGrCur=[];          % current
firstTime=0;
Ngold=0;                % Number of gold iter left
maxGrPhase1=0;          % for enter phase 2 (for gr==3)
prevMaxGrPhase1=0;      % for enter phase 2 (for gr==3)
maxBigGr_N=0;           % for enter phase 2 (for gr>3)
maxPin_N=0;

newMax=0;
Pin=0;
gotPin=0;               % is in this cycle got Pin (explotation state)
curSol7=1;              % current sol for 7 point alg
noOfsol=1;              % for 7 point alg
i_order_update=0;       % Should update i_order counter after get out fron Ngold loops
             

while(iter<N_BEEM),
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Statistic
    if(chkStatis & calcPin(outCDF,max(iter,1),maxInGr_N,maxNoGrOut,1)),
    tmp=sum(ismember(allIn,maxInGr));
    if (tmp/size(allIn,1) >=0.75),
        if( ~RANSACres(1,1) ),
            tmp=sum(ismember(outDegIn,maxInGr));
            if (tmp),
                RANSACres(1,1)=iter;
            end
        end
        if( ~RANSACres(1,2) ),
            tmp=sum(ismember(outDegIn,maxInGr));
            if (tmp/size(outDegIn,1) >=0.5),
                RANSACres(1,2)=iter;
            end
        end
        if( ~RANSACres(1,3) ),
            tmp=sum(ismember(outDegIn,maxInGr));
            if (tmp/size(outDegIn,1) >=0.75),
                RANSACres(1,3)=iter;
            end
        end
        if( ~RANSACres(1,4) ),
            tmp=sum(ismember(allIn,maxInGr));
            if (tmp/size(allIn,1) >=0.75),
                RANSACres(1,4)=iter;
            end
        end
    end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if (Ngold>0 & NgrSam==7)
    curSol7=1;
    noOfsol=1;
end
    
if ( ((NgrSam==7) & (curSol7==1)) | (NgrSam~=7) ),
    iterDD=iterDD+1;
end
gotPin=0;

%%%%%%%%%%%%%%%%%
if (Pin==1),
%%%%%%%%%%%%%%%%%
%gotPin=1;
    if (Ngold==0),
        if(i_order_update),
            i_order_update=0;
            i_order=0;
            N_BEEM=iter+N_order;
            maxPin_N=NgrSam*2;
        end
              
        i_order=i_order+1;
    
        if (i_order>N_order), break;end                 %%%% Exist search loop!!!
        
            iter=iter+1;
            gotPin=1;
            maxInProb=probLiran(maxInGr);
            maxInProb=maxInProb/max(maxInProb);
            gr_sz=max((floor(maxInGr_N/2)+1),NgrSam);             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            gr_sz=min(gr_sz,20);             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            gr=zeros(gr_sz,1);            
            taken=zeros(maxInGr_N,1);

                for i=1:(gr_sz-1),                          %% untill gr_sz-1
                    OK=0;
                    while (~OK),
                        sam=floor(rand*maxInGr_N+1);
                        if(taken(sam)==0),
                            if (rand<=maxInProb((sam))),OK=1;end
                        end
                    end
                    taken(sam)=1;
                    gr(i)=maxInGr(sam);
                end
            
                gr(gr_sz)=vec_order(i_order);
                             

    else    % Ngold==0
            Ngold=Ngold-1;
            maxInProb=probLiran(maxInGrCur);
            maxInProb=maxInProb/max(maxInProb);
            gr_sz=max((floor(maxInGrCur_N/2)),NgrSam);             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            gr_sz=min(gr_sz,20);
            gr=zeros(gr_sz,1);
            stop=1;
            taken=zeros(maxInGrCur_N,1);
            while(stop),
                stop=0;
                for i=1:(gr_sz-1),
                    OK=0;
                    while (~OK),
                        sam=floor(rand*maxInGrCur_N+1);
                        if(taken(sam)==0),
                            if (rand<=maxInProb((sam))),OK=1;end
                        end
                    end
                    taken(sam)=1;
                    gr(i)=maxInGrCur(sam);
                end
            end
            gr(gr_sz)=vec_order(i_order);
    end     % Ngold==0
else    %% if Pin==1    

%%%

if (( (NgrSam==7) & (noOfsol==1)) | (NgrSam~=7)),   %%%->
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Ngold
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



    if (Ngold>0),
        Ngold=Ngold-1;

        
%        maxInGrCur_N
%        maxInGrCur

            maxInProb=probLiran(maxInGrCur);
            maxInProb=maxInProb/max(maxInProb);
            gr_sz=max((floor(maxInGrCur_N/2)),NgrSam);             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            gr_sz=min(gr_sz,14);
            gr=zeros(gr_sz,1);
            stop=1;
            taken=zeros(maxInGrCur_N,1);
            while(stop),
                stop=0;
                for i=1:(gr_sz),
                    OK=0;
                    while (~OK),
                        sam=floor(rand*maxInGrCur_N+1);
                        if(taken(sam)==0),
                            if (rand<=maxInProb((sam))),OK=1;end
                        end
                    end
                    taken(sam)=1;
                    gr(i)=maxInGrCur(sam);
                end
            end
    else

        %% Calc Pin
        Pin=calcPin(outCDF,max(iter,1),maxInGr_N,maxNoGrOut,updatePin);
        if Pin==1, 
            [vec_order, N_order, i_order]=update_order(maxInGr, probLiran);
            N_BEEM=iter+N_order;
            continue;
        end

        iter=iter+1;
        
%Pin=0;

        if (rand >= Pin) | (floor(maxInGr_N/2) <NgrSam),
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Not Pin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            iterGlobal=iterGlobal+1;

            gr=zeros(NgrSam,1);
            stop=1;
            while(stop),
                stop=0;
                for i=1:NgrSam,
                    OK=0;
                    while (~OK),
                        sam=floor(rand*N+1);
                        if (rand<=probLiran(sam)),OK=1;end
                    end
                    gr(i)=sam;
                end
    %            gr=floor(rand(4,1)*N+1);
                gr=sort(gr);
                for i=1:(NgrSam-1), if (gr(i)==gr(i+1)), stop=1;end;end;
            end
        else

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Pin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
        gotPin=1;
            outCor=ones(N,1);
            outCor(maxInGr)=0;
            outCor=find(outCor);
            outProb=probLiran(outCor);
            outProb=outProb/max(outProb);
            maxInProb=probLiran(maxInGr);
            maxInProb=maxInProb/max(maxInProb);
            gr_sz=max((floor(maxInGr_N/2)+1),NgrSam);             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
gr_sz=min(gr_sz,14);             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%gr_sz=3;
%maxInProb=maxInProb./maxInProb;
%gr_sz=max(round(gr_sz*rand),3);
            gr=zeros(gr_sz,1);            
            stop=1;
            taken=zeros(maxInGr_N,1);
            while(stop),
                stop=0;
                for i=1:(gr_sz-1),                          %% untill gr_sz-1
                    OK=0;
                    while (~OK),
                        sam=floor(rand*maxInGr_N+1);
                        if(taken(sam)==0),
                            if (rand<=maxInProb((sam))),OK=1;end
                        end
                    end
                    taken(sam)=1;
                    gr(i)=maxInGr(sam);
                end
            
                
                NN=size(outCor,1);
                OK=0;
                while (~OK),
                    sam=floor(rand*NN+1);
                    if (rand<=outProb(sam)),OK=1;end
                end
                gr(gr_sz)=outCor(sam);
                             
%                gr=sort(gr);
%                for i=1:(NgrSam-1), if (gr(i)==gr(i+1)), stop=1;end;end;
            end
            
            
        end
    end
else        % if ((NgrSam==7 & noOfsol~=curSol7) | (NgrSam~=7))
    curSol7=curSol7+1;
end
    

end     %% if Pin==1

    if (size(gr,1)>=8),
        if (gotPin),
            F=epoint([match(gr,:); match_g(gr(gr_sz),:); match_g1(gr(gr_sz),:); match_g2(gr(gr_sz),:)]);
        else
            F=epoint([match(gr,:)]);
        end
    else
        if ( (NgrSam==7) & (size(gr,1)==7) ),
            [F,noOfsol]=spoint([match(gr,:)],curSol7,0);
            if (noOfsol==curSol7),
                noOfsol=1;
                curSol7=1;
            end
        else
            F=epoint([match(gr,:); match_g(gr,:); match_g1(gr,:); match_g2(gr,:)],1);
        end
    end

    InGr_N=0;
            InGr=zeros(1,N);
            for i=1:N,
                v1=F*[match(i,1:2),1]';
                v2=F'*[match(i,3:4),1]';
                if ([match(i,3:4),1]*F*[match(i,1:2),1]')^2/(v1(1)^2+v1(2)^2+v2(1)^2+v2(2)^2)<ransac_tr,
                    v1=F*[match_g(i,1:2),1]';
                    v2=F'*[match_g(i,3:4),1]';
                    v1_1=F*[match_g1(i,1:2),1]';
                    v2_1=F'*[match_g1(i,3:4),1]';
                    v1_2=F*[match_g2(i,1:2),1]';
                    v2_2=F'*[match_g2(i,3:4),1]';

%                    if ([match_g(i,3:4),1]*F*[match_g(i,1:2),1]')^2/(v1(1)^2+v1(2)^2+v2(1)^2+v2(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)+rr2(i,2))/2)^2 & ([match_g1(i,3:4),1]*F*[match_g1(i,1:2),1]')^2/(v1_1(1)^2+v1_1(2)^2+v2_1(1)^2+v2_1(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)+rr2(i,2))/2)^2,
                    if (constraintAll3SIFTpoints),
                        if ([match_g(i,3:4),1]*F*[match_g(i,1:2),1]')^2/(v1(1)^2+v1(2)^2+v2(1)^2+v2(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)*rr2(i,2))) & ([match_g1(i,3:4),1]*F*[match_g1(i,1:2),1]')^2/(v1_1(1)^2+v1_1(2)^2+v2_1(1)^2+v2_1(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)*rr2(i,2))) & ([match_g2(i,3:4),1]*F*[match_g2(i,1:2),1]')^2/(v1_2(1)^2+v1_2(2)^2+v2_2(1)^2+v2_2(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)*rr2(i,2))),
                            InGr_N=InGr_N+1;
                            InGr(InGr_N)=i;
                        end
                    else
                        InGr_N=InGr_N+1;
                        InGr(InGr_N)=i;
                    end
                end
            end
    InGr=InGr(1:InGr_N);


    %%%%%%%%%%%%%%%%%%%%%%% Clean repeated corners    
    if InGr_N>1, 
        repeatCount=0;
        chD=[match(InGr,1:4) InGr'];
        chD=sortrows(chD,1);
        for i=1:(InGr_N-1),
            if ( (chD(i,1)==chD(i+1,1))&((chD(i,2)==chD(i+1,2))) ),
                repeatCount=repeatCount+1;
                chD(i,5)=0;
            end
        end
        chD=sortrows(chD,3);
        for i=1:(InGr_N-1),
            if ( (chD(i,3)==chD(i+1,3))&((chD(i,4)==chD(i+1,4))) ),
                if ~( (chD(i,1)==chD(i+1,1))&((chD(i,2)==chD(i+1,2))) )
                    repeatCount=repeatCount+1;
                    chD(i,5)=0;
                end
            end
        end

        InGr=chD(find(chD(:,5)), 5);
        InGr_N=InGr_N-repeatCount;        

    end

    if ( (InGr_N>(maxGrPhase1)&(InGr_N>NgrSam)& (size(gr,1)==NgrSam)) | ((InGr_N>maxBigGr_N)&(InGr_N>NgrSam)&(size(gr,1)>NgrSam)&(Pin<1)) | ((InGr_N>=maxPin_N)&(InGr_N>NgrSam)&(size(gr,1)>NgrSam)&(Pin==1))),
        
    fprintf('*');
        
        if ( (InGr_N>(maxGrPhase1))&(Ngold<=0)&(size(gr,1)==NgrSam) ),
            prevMaxGrPhase1=maxGrPhase1;
            maxGrPhase1=InGr_N;
            if (InGr_N>=NgrSam*2 ),
                Ngold=MaxGoldIter;
                maxInGrCur_N=InGr_N;
                maxInGrCur=InGr;
                firstTime=1;
            end
        end

        if ( (InGr_N>maxBigGr_N) & (size(gr,1)>NgrSam) ),
            maxBigGr_N=InGr_N;
            Ngold=MaxGoldIter;
            maxInGrCur_N=InGr_N;
            maxInGrCur=InGr;
            firstTime=1;
        end

        
        if ( (InGr_N>maxPin_N) & (size(gr,1)>NgrSam) & (Pin==1) ),
            maxPin_N=InGr_N;
            Ngold=MaxGoldIter;
            maxInGrCur_N=InGr_N;
            maxInGrCur=InGr;
            firstTime=1;
        end


        %% Phase 2
        
        cont=1;
        prevInGr_N=InGr_N;
        while(cont),
            if (LO),
                if (size(InGr,1)>=8),
                    F=epoint([match(InGr,:)]);
                else
                    if (NgrSam<7),
                        F=epoint([match(InGr,:); match_g(InGr,:); match_g1(InGr,:); match_g2(InGr,:)]);
                    end
                end
            end
        
            InGr_N=0;
            InGr=zeros(1,N);
            for i=1:N,
                v1=F*[match(i,1:2),1]';
                v2=F'*[match(i,3:4),1]';
                if ([match(i,3:4),1]*F*[match(i,1:2),1]')^2/(v1(1)^2+v1(2)^2+v2(1)^2+v2(2)^2)<ransac_tr,
                    v1=F*[match_g(i,1:2),1]';
                    v2=F'*[match_g(i,3:4),1]';
                    v1_1=F*[match_g1(i,1:2),1]';
                    v2_1=F'*[match_g1(i,3:4),1]';
                    v1_2=F*[match_g2(i,1:2),1]';
                    v2_2=F'*[match_g2(i,3:4),1]';

%                    if ([match_g(i,3:4),1]*F*[match_g(i,1:2),1]')^2/(v1(1)^2+v1(2)^2+v2(1)^2+v2(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)+rr2(i,2))/2)^2 & ([match_g1(i,3:4),1]*F*[match_g1(i,1:2),1]')^2/(v1_1(1)^2+v1_1(2)^2+v2_1(1)^2+v2_1(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)+rr2(i,2))/2)^2,
                    if (constraintAll3SIFTpoints),
                        if ([match_g(i,3:4),1]*F*[match_g(i,1:2),1]')^2/(v1(1)^2+v1(2)^2+v2(1)^2+v2(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)*rr2(i,2))) & ([match_g1(i,3:4),1]*F*[match_g1(i,1:2),1]')^2/(v1_1(1)^2+v1_1(2)^2+v2_1(1)^2+v2_1(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)*rr2(i,2)))  & ([match_g2(i,3:4),1]*F*[match_g2(i,1:2),1]')^2/(v1_2(1)^2+v1_2(2)^2+v2_2(1)^2+v2_2(2)^2)<ransac_tr*multSIFTd*((rr1(i,2)*rr2(i,2))),
                            InGr_N=InGr_N+1;
                            InGr(InGr_N)=i;
                        end
                    else
                        InGr_N=InGr_N+1;
                        InGr(InGr_N)=i;
                    end
                end
            end
            InGr=InGr(1:InGr_N);
    if InGr_N>1, 
    repeatCount=0;
    chD=[match(InGr,1:4), InGr'];
    chD=sortrows(chD,1);
    for i=1:(InGr_N-1),
        if ( (chD(i,1)==chD(i+1,1))&((chD(i,2)==chD(i+1,2))) ),
            repeatCount=repeatCount+1;
            chD(i,5)=0;
        end
    end
    chD=sortrows(chD,3);
    for i=1:(InGr_N-1),
        if ( (chD(i,3)==chD(i+1,3))&((chD(i,4)==chD(i+1,4))) ),
            if ~( (chD(i,1)==chD(i+1,1))&((chD(i,2)==chD(i+1,2))) )
                repeatCount=repeatCount+1;
                chD(i,5)=0;
            end
        end
    end
    
    InGr=chD(find(chD(:,5)),5);
    InGr_N=InGr_N-repeatCount;

    end    
            if (prevInGr_N<InGr_N),
                prevInGr_N=InGr_N;
%                fprintf('*iter %d/%d,  gr %d/%d=%0.2g\n',iter,N_BEEM,maxNoGr,N,maxNoGr/N);
            else
                cont=0;
            end         

            if (InGr_N>(maxInGrCur_N)),
                maxInGrCur_N=InGr_N;
                maxInGrCur=InGr;
            end

            if (InGr_N>(maxInGr_N)),
                newMax=1;
                maxInGr_N=InGr_N;
                maxInGr=InGr;
                
                firstTime=0;
                if Pin==1, 
                    [vec_order, N_order, i_order_update]=update_order(maxInGr, probLiran);
                    N_BEEM=iter+N_order;
                end
            end
        end % of while
        
        if newMax,
            newMax=0;
            
%            PinBak=Pin;     % for printing
            Pin_print=calcPin(outCDF,iter,maxInGr_N,maxNoGrOut,updatePin);
            
            if(~updatePin),
                tmp=min(round(log(1-0.99)/log(1-(maxInGr_N/N)^NgrSam)),N_BEEM);
                if (tmp>0),
                    N_BEEM=tmp;
                end
            end
            
            if (firstTime), fprintf('*iter %d/%d,  gr %d/%d=%0.2g \n',iter,N_BEEM,maxInGr_N,N,maxInGr_N/N);
            else            fprintf(' iter %d/%d,  gr %d/%d=%0.2g \n',iter,N_BEEM,maxInGr_N,N,maxInGr_N/N);end

            if maxInGr_N>=Ngoal,
                iterDD
                return;
            end

        end
        firstTime=0;
    end
    
    if (mod(iter,1000)==0),
        fprintf('=iter %d/%d,  gr %d/%d=%0.2g \n',iter,N_BEEM,maxInGr_N,N,maxInGr_N/N);
        pause(0.01);
    end
    
end % of while   

%Pin

InIndex=maxInGr;

% save InIndex InIndex;
% save match_g match_g;
% save match_g1 match_g1;
% save match_g2 match_g2;
if(size(InIndex,1)>=8),
    F=epoint(match(InIndex,:));
else
    F=zeros(3,3);
end
% save F F;
% save iterGlobal iterGlobal;

if(chkStatis),
    RANSACres(1,5)=iter;

    tmp=sum(ismember(allIn,maxInGr));
    RANSACres(1,6)=tmp;

    tmp=sum(ismember(outDegIn,maxInGr));
    RANSACres(1,7)=tmp;
end
